@@ -29,6 +29,7 @@
 
 #include "kconfigbackend.h"
 #include "kconfiggroup.h"
+#include <kde_file.h>
 #include <kstringhandler.h>
 #include <klocale.h>
 #include <kstandarddirs.h>
@@ -53,50 +54,45 @@
 bool KConfigPrivate::mappingsRegistered=false;
 
 KConfigPrivate::KConfigPrivate(const KComponentData &componentData_, KConfig::OpenFlags flags,
-           const char* resource)
+                               const char* resource)
     : openFlags(flags), resourceType(resource), mBackend(0),
       bDynamicBackend(true),  bDirty(false), bReadDefaults(false),
-      bFileImmutable(false), bForceGlobal(false), componentData(componentData_),
-      configState(KConfigBase::NoAccess)
+      bFileImmutable(false), bForceGlobal(false), bSuppressGlobal(false),
+      componentData(componentData_), configState(KConfigBase::NoAccess)
 {
-    sGlobalFileName = componentData.dirs()->saveLocation("config") +
-                          QString::fromLatin1("kdeglobals");
-    if (wantGlobals()) {
-        const KStandardDirs *const dirs = componentData.dirs();
-        foreach(const QString& dir, dirs->findAllResources("config", QLatin1String("kdeglobals")) +
-                                    dirs->findAllResources("config", QLatin1String("system.kdeglobals")))
-            globalFiles.push_front(dir);
-    }
-    const QString etc_kderc =
+    sGlobalFileName = componentData.dirs()->saveLocation("config", QString(), false) + QLatin1String("kdeglobals");
+
+    static int use_etc_kderc = -1;
+    if (use_etc_kderc < 0)
+        use_etc_kderc = getenv("KDE_SKIP_KDERC") != 0 ? 0 : 1; // for unit tests
+    if (use_etc_kderc) {
+
+        etc_kderc =
 #ifdef Q_WS_WIN
-        QFile::decodeName( qgetenv("WINDIR") + "/kde4rc" );
+            QFile::decodeName( qgetenv("WINDIR") + "/kde4rc" );
 #else
-        QLatin1String("/etc/kde4rc");
+            QLatin1String("/etc/kde4rc");
 #endif
-    KEntryMap tmp;
-    // first entry is always /etc/kderc or empty if cannot read
-    if (KStandardDirs::checkAccess(etc_kderc, R_OK)) {
-        if (!globalFiles.contains(etc_kderc))
-            globalFiles.push_front(etc_kderc);
-
-        if (!mappingsRegistered) {
-            KSharedPtr<KConfigBackend> backend = KConfigBackend::create(componentData, etc_kderc, QLatin1String("INI"));
-            backend->parseConfig( "en_US", tmp, KConfigBackend::ParseDefaults);
+        if (!KStandardDirs::checkAccess(etc_kderc, R_OK)) {
+            etc_kderc.clear();
         }
-    } else {
-        globalFiles.push_front(QString());
-        mappingsRegistered = true;
     }
 
-    if (!mappingsRegistered) {
-        const QString kde4rc(QDir::home().filePath(".kde4rc"));
-        if (KStandardDirs::checkAccess(kde4rc, R_OK)) {
-            KSharedPtr<KConfigBackend> backend = KConfigBackend::create(componentData, kde4rc, QLatin1String("INI"));
-            backend->parseConfig( "en_US", tmp, KConfigBackend::ParseOptions());
-        }
-        KConfigBackend::registerMappings(tmp);
-        mappingsRegistered = true;
-    }
+//    if (!mappingsRegistered) {
+//        KEntryMap tmp;
+//        if (!etc_kderc.isEmpty()) {
+//            KSharedPtr<KConfigBackend> backend = KConfigBackend::create(componentData, etc_kderc, QLatin1String("INI"));
+//            backend->parseConfig( "en_US", tmp, KConfigBackend::ParseDefaults);
+//        }
+//        const QString kde4rc(QDir::home().filePath(".kde4rc"));
+//        if (KStandardDirs::checkAccess(kde4rc, R_OK)) {
+//            KSharedPtr<KConfigBackend> backend = KConfigBackend::create(componentData, kde4rc, QLatin1String("INI"));
+//            backend->parseConfig( "en_US", tmp, KConfigBackend::ParseOptions());
+//        }
+//        KConfigBackend::registerMappings(tmp);
+//        mappingsRegistered = true;
+//    }
+
     setLocale(KGlobal::hasLocale() ? KGlobal::locale()->language() : KLocale::defaultLanguage());
 }
 
@@ -104,10 +100,6 @@ KConfigPrivate::KConfigPrivate(const KComponentData &componentData_, KConfig::Op
 bool KConfigPrivate::lockLocal()
 {
     if (mBackend) {
-        if (fileName == QLatin1String("kdeglobals")) { // we don't want to lock "kdeglobals" twice
-            if (wantGlobals()) // "kdeglobals" will be locked with the global lock
-                return true; // so pretend we locked it here
-        }
         return mBackend->lock(componentData);
     }
     // anonymous object - pretend we locked it
@@ -160,6 +152,92 @@ void KConfigPrivate::copyGroup(const QByteArray& source, const QByteArray& desti
     }
 }
 
+QString KConfigPrivate::expandString(const QString& value)
+{
+    QString aValue = value;
+
+    // check for environment variables and make necessary translations
+    int nDollarPos = aValue.indexOf( QLatin1Char('$') );
+    while( nDollarPos != -1 && nDollarPos+1 < aValue.length()) {
+        // there is at least one $
+        if( aValue[nDollarPos+1] == QLatin1Char('(') ) {
+            int nEndPos = nDollarPos+1;
+            // the next character is not $
+            while ( (nEndPos <= aValue.length()) && (aValue[nEndPos]!=QLatin1Char(')')) )
+                nEndPos++;
+            nEndPos++;
+            QString cmd = aValue.mid( nDollarPos+2, nEndPos-nDollarPos-3 );
+
+            QString result;
+            QByteArray oldpath = qgetenv( "PATH" );
+            QByteArray newpath;
+            if (KGlobal::hasMainComponent()) {
+                newpath = QFile::encodeName(KGlobal::dirs()->resourceDirs("exe").join(QChar::fromLatin1(KPATH_SEPARATOR)));
+                if (!newpath.isEmpty() && !oldpath.isEmpty())
+                    newpath += KPATH_SEPARATOR;
+            }
+            newpath += oldpath;
+            setenv( "PATH", newpath, 1/*overwrite*/ );
+// FIXME: wince does not have pipes
+#ifndef _WIN32_WCE
+            FILE *fs = popen(QFile::encodeName(cmd).data(), "r");
+            if (fs) {
+                QTextStream ts(fs, QIODevice::ReadOnly);
+                result = ts.readAll().trimmed();
+                pclose(fs);
+            }
+#endif
+            setenv( "PATH", oldpath, 1/*overwrite*/ );
+            aValue.replace( nDollarPos, nEndPos-nDollarPos, result );
+            nDollarPos += result.length();
+        } else if( aValue[nDollarPos+1] != QLatin1Char('$') ) {
+            int nEndPos = nDollarPos+1;
+            // the next character is not $
+            QString aVarName;
+            if ( aValue[nEndPos] == QLatin1Char('{') ) {
+                while ( (nEndPos <= aValue.length()) && (aValue[nEndPos] != QLatin1Char('}')) )
+                    nEndPos++;
+                nEndPos++;
+                aVarName = aValue.mid( nDollarPos+2, nEndPos-nDollarPos-3 );
+            } else {
+                while ( nEndPos <= aValue.length() &&
+                        (aValue[nEndPos].isNumber() ||
+                        aValue[nEndPos].isLetter() ||
+                        aValue[nEndPos] == QLatin1Char('_') ) )
+                    nEndPos++;
+                aVarName = aValue.mid( nDollarPos+1, nEndPos-nDollarPos-1 );
+            }
+            QString env;
+            if (!aVarName.isEmpty()) {
+#ifdef Q_OS_WIN
+                if (aVarName == QLatin1String("HOME"))
+                    env = QDir::homePath();
+                else
+#endif
+                {
+                    QByteArray pEnv = qgetenv( aVarName.toLatin1() );
+                    if( !pEnv.isEmpty() )
+                    // !!! Sergey A. Sukiyazov <corwin@micom.don.ru> !!!
+                    // An environment variable may contain values in 8bit
+                    // locale specified encoding or UTF8 encoding
+                        env = KStringHandler::from8Bit( pEnv );
+                }
+                aValue.replace(nDollarPos, nEndPos-nDollarPos, env);
+                nDollarPos += env.length();
+            } else
+                aValue.remove( nDollarPos, nEndPos-nDollarPos );
+        } else {
+            // remove one of the dollar signs
+            aValue.remove( nDollarPos, 1 );
+            nDollarPos++;
+        }
+        nDollarPos = aValue.indexOf( QLatin1Char('$'), nDollarPos );
+    }
+
+    return aValue;
+}
+
+
 KConfig::KConfig( const QString& file, OpenFlags mode,
                   const char* resourceType)
   : d_ptr(new KConfigPrivate(KGlobal::mainComponent(), mode, resourceType))
@@ -213,16 +291,18 @@ const KComponentData& KConfig::componentData() const
 QStringList KConfig::groupList() const
 {
     Q_D(const KConfig);
-    QStringList groups;
+    QSet<QString> groups;
 
     for (KEntryMap::ConstIterator entryMapIt( d->entryMap.constBegin() ); entryMapIt != d->entryMap.constEnd(); ++entryMapIt) {
-        const QByteArray group = entryMapIt.key().mGroup;
-        if (entryMapIt.key().mKey.isNull() && !group.isEmpty() &&
-            group != "<default>" && group != "$Version" && !group.contains('\x1d'))
-            groups << QString::fromUtf8(group);
+        const KEntryKey& key = entryMapIt.key();	
+        const QByteArray group = key.mGroup;
+        if (key.mKey.isNull() && !group.isEmpty() && group != "<default>" && group != "$Version") {
+            const QString groupname = QString::fromUtf8(group);
+            groups << groupname.left(groupname.indexOf(QLatin1Char('\x1d')));
+        }
     }
 
-    return groups;
+    return groups.toList();
 }
 
 QStringList KConfigPrivate::groupList(const QByteArray& group) const
@@ -230,24 +310,53 @@ QStringList KConfigPrivate::groupList(const QByteArray& group) const
     QByteArray theGroup = group + '\x1d';
     QSet<QString> groups;
 
-    for (KEntryMap::ConstIterator entryMapIt( entryMap.constBegin() ); entryMapIt != entryMap.constEnd(); ++entryMapIt)
-        if (entryMapIt.key().mKey.isNull() && entryMapIt.key().mGroup.startsWith(theGroup))
-        {
-            QString groupname = QString::fromUtf8(entryMapIt.key().mGroup.mid(theGroup.length()));
-            groups << groupname.left(groupname.indexOf('\x1d'));
+    for (KEntryMap::ConstIterator entryMapIt( entryMap.constBegin() ); entryMapIt != entryMap.constEnd(); ++entryMapIt) {
+        const KEntryKey& key = entryMapIt.key();	
+        if (key.mKey.isNull() && key.mGroup.startsWith(theGroup)) {
+            const QString groupname = QString::fromUtf8(key.mGroup.mid(theGroup.length()));
+            groups << groupname.left(groupname.indexOf(QLatin1Char('\x1d')));
         }
+    }
 
     return groups.toList();
 }
 
-QStringList KConfig::keyList(const QString& aGroup) const
+// List all sub groups, including subsubgroups
+QSet<QByteArray> KConfigPrivate::allSubGroups(const QByteArray& parentGroup) const
 {
-    Q_D(const KConfig);
-    QStringList keys;
-    const QByteArray theGroup(aGroup.isEmpty() ? "<default>" : aGroup.toUtf8());
+    QSet<QByteArray> groups;
+    QByteArray theGroup = parentGroup + '\x1d';
+    groups << parentGroup;
 
-    const KEntryMapConstIterator theEnd = d->entryMap.constEnd();
-    KEntryMapConstIterator it = d->entryMap.findEntry(theGroup);
+    for (KEntryMap::const_iterator entryMapIt = entryMap.begin(); entryMapIt != entryMap.end(); ++entryMapIt) {
+        const KEntryKey& key = entryMapIt.key();
+        if (key.mKey.isNull() && key.mGroup.startsWith(theGroup)) {
+            groups << key.mGroup;
+        }
+    }
+    return groups;
+}
+
+bool KConfigPrivate::hasNonDeletedEntries(const QByteArray& group) const
+{
+    const QSet<QByteArray> allGroups = allSubGroups(group);
+
+    Q_FOREACH(const QByteArray& aGroup, allGroups) {
+        // Could be optimized, let's use the slow way for now
+        // Check for any non-deleted entry
+        if (!keyListImpl(aGroup).isEmpty())
+            return true;
+    }
+    return false;
+}
+
+
+QStringList KConfigPrivate::keyListImpl(const QByteArray& theGroup) const
+{
+    QStringList keys;
+
+    const KEntryMapConstIterator theEnd = entryMap.constEnd();
+    KEntryMapConstIterator it = entryMap.findEntry(theGroup);
     if (it != theEnd) {
         ++it; // advance past the special group entry marker
 
@@ -263,6 +372,13 @@ QStringList KConfig::keyList(const QString& aGroup) const
     return keys;
 }
 
+QStringList KConfig::keyList(const QString& aGroup) const
+{
+    Q_D(const KConfig);
+    const QByteArray theGroup(aGroup.isEmpty() ? "<default>" : aGroup.toUtf8());
+    return d->keyListImpl(theGroup);
+}
+
 QMap<QString,QString> KConfig::entryMap(const QString& aGroup) const
 {
     Q_D(const KConfig);
@@ -280,8 +396,13 @@ QMap<QString,QString> KConfig::entryMap(const QString& aGroup) const
                 const QString key = QString::fromUtf8(it.key().mKey.constData());
                 // the localized entry should come first, so don't overwrite it
                 // with the non-localized entry
-                if (!theMap.contains(key))
-                    theMap.insert(key,QString::fromUtf8(it->mValue.constData()));
+                if (!theMap.contains(key)) {
+                    if (it->bExpand) {
+                        theMap.insert(key,KConfigPrivate::expandString(QString::fromUtf8(it->mValue.constData())));
+                    } else {
+                        theMap.insert(key,QString::fromUtf8(it->mValue.constData()));
+                    }
+                }
             }
         }
     }
@@ -289,11 +410,15 @@ QMap<QString,QString> KConfig::entryMap(const QString& aGroup) const
     return theMap;
 }
 
+// TODO KDE5: return a bool value
 void KConfig::sync()
 {
     Q_D(KConfig);
 
-    Q_ASSERT(!isImmutable() && !name().isEmpty()); // can't write to an immutable or anonymous file.
+    if (isImmutable() || name().isEmpty()) {
+        // can't write to an immutable or anonymous file.
+        return;
+    }
 
     if (d->bDirty && d->mBackend) {
         const QByteArray utf8Locale(locale().toUtf8());
@@ -334,6 +459,8 @@ void KConfig::sync()
             }
             if (!tmp->writeConfig(utf8Locale, d->entryMap, KConfigBackend::WriteGlobal, d->componentData)) {
                 d->bDirty = true;
+                // TODO KDE5: return false? (to tell the app that writing wasn't possible, e.g.
+                // config file is immutable or disk full)
             }
             if (tmp->isLocked()) {
                 tmp->unlock();
@@ -343,6 +470,8 @@ void KConfig::sync()
         if (writeLocals) {
             if (!d->mBackend->writeConfig(utf8Locale, d->entryMap, KConfigBackend::WriteOptions(), d->componentData)) {
                 d->bDirty = true;
+                // TODO KDE5: return false? (to tell the app that writing wasn't possible, e.g.
+                // config file is immutable or disk full)
             }
         }
         if (d->mBackend->isLocked()) {
@@ -365,10 +494,10 @@ void KConfig::markAsClean()
 void KConfig::checkUpdate(const QString &id, const QString &updateFile)
 {
     const KConfigGroup cg(this, "$Version");
-    const QString cfg_id = updateFile+':'+id;
-    QStringList ids = cg.readEntry("update_info", QStringList());
+    const QString cfg_id = updateFile+QLatin1Char(':')+id;
+    const QStringList ids = cg.readEntry("update_info", QStringList());
     if (!ids.contains(cfg_id)) {
-        KToolInvocation::kdeinitExecWait("kconf_update", QStringList() << "--check" << updateFile);
+        KToolInvocation::kdeinitExecWait(QString::fromLatin1("kconf_update"), QStringList() << QString::fromLatin1("--check") << updateFile);
         reparseConfiguration();
     }
 }
@@ -410,29 +539,28 @@ void KConfigPrivate::changeFileName(const QString& name, const char* type)
                     resourceType = type; // only change it if it's not empty
                 file = KStandardDirs::locateLocal(resourceType, fileName, false, componentData);
             }
-        } else if (wantGlobals()) { // accessing "kdeglobals"
+        } else if (wantGlobals()) { // accessing "kdeglobals" - XXX used anywhere?
             resourceType = "config";
             fileName = QLatin1String("kdeglobals");
             file = sGlobalFileName;
-        }
-    } else if (QDir::isAbsolutePath(fileName))
+        } // else anonymous config.
+        // KDE5: remove these magic overloads
+    } else if (QDir::isAbsolutePath(fileName)) {
+        fileName = KStandardDirs::realFilePath(fileName);
         file = fileName;
-    else {
+    } else {
         if (type && *type)
             resourceType = type; // only change it if it's not empty
         file = KStandardDirs::locateLocal(resourceType, fileName, false, componentData);
-
-        if (fileName == QLatin1String("kdeglobals"))
-            openFlags |= KConfig::IncludeGlobals;
     }
 
-    bForceGlobal = (fileName == QLatin1String("kdeglobals"));
-
     if (file.isEmpty()) {
         openFlags = KConfig::SimpleConfig;
         return;
     }
 
+    bSuppressGlobal = (file == sGlobalFileName);
+
     if (bDynamicBackend || !mBackend) // allow dynamic changing of backend
         mBackend = KConfigBackend::create(componentData, file);
     else
@@ -444,6 +572,10 @@ void KConfigPrivate::changeFileName(const QString& name, const char* type)
 void KConfig::reparseConfiguration()
 {
     Q_D(KConfig);
+    if (d->fileName.isEmpty()) {
+        return;
+    }
+
     // Don't lose pending changes
     if (!d->isReadOnly() && d->bDirty)
         sync();
@@ -459,8 +591,23 @@ void KConfig::reparseConfiguration()
     d->parseConfigFiles();
 }
 
+
+QStringList KConfigPrivate::getGlobalFiles() const
+{
+    const KStandardDirs *const dirs = componentData.dirs();
+    QStringList globalFiles;
+    foreach (const QString& dir1, dirs->findAllResources("config", QLatin1String("kdeglobals")))
+        globalFiles.push_front(dir1);
+    foreach (const QString& dir2, dirs->findAllResources("config", QLatin1String("system.kdeglobals")))
+        globalFiles.push_front(dir2);
+    if (!etc_kderc.isEmpty())
+        globalFiles.push_front(etc_kderc);
+    return globalFiles;
+}
+
 void KConfigPrivate::parseGlobalFiles()
 {
+    const QStringList globalFiles = getGlobalFiles();
 //    qDebug() << "parsing global files" << globalFiles;
 
     // TODO: can we cache the values in etc_kderc / other global files
@@ -479,21 +626,23 @@ void KConfigPrivate::parseGlobalFiles()
 
 void KConfigPrivate::parseConfigFiles()
 {
-    if (fileName == QLatin1String("kdeglobals") && wantGlobals())
-        return; // already parsed in parseGlobalFiles()
-
     // can only read the file if there is a backend and a file name
     if (mBackend && !fileName.isEmpty()) {
 
         bFileImmutable = false;
+
         QList<QString> files;
-
-        if (wantDefaults())
-            foreach (const QString& f, componentData.dirs()->findAllResources(resourceType, fileName))
-                files.prepend(f);
-        else
+        if (wantDefaults()) {
+            if (bSuppressGlobal) {
+                files = getGlobalFiles();
+            } else {
+                foreach (const QString& f, componentData.dirs()->findAllResources(
+                                                    resourceType, fileName))
+                    files.prepend(f);
+            }
+        } else {
             files << mBackend->filePath();
-
+        }
         if (!isSimple())
             files = extraFiles.toList() + files;
 
@@ -594,17 +743,21 @@ bool KConfig::isGroupImmutableImpl(const QByteArray& aGroup) const
     return isImmutable() || d->entryMap.getEntryOption(aGroup, 0, 0, KEntryMap::EntryImmutable);
 }
 
+#ifndef KDE_NO_DEPRECATED
 void KConfig::setForceGlobal(bool b)
 {
     Q_D(KConfig);
     d->bForceGlobal = b;
 }
+#endif
 
+#ifndef KDE_NO_DEPRECATED
 bool KConfig::forceGlobal() const
 {
     Q_D(const KConfig);
     return d->bForceGlobal;
 }
+#endif
 
 KConfigGroup KConfig::groupImpl(const QByteArray &group)
 {
@@ -634,18 +787,9 @@ void KConfig::deleteGroupImpl(const QByteArray &aGroup, WriteConfigFlags flags)
     Q_D(KConfig);
     KEntryMap::EntryOptions options = convertToOptions(flags)|KEntryMap::EntryDeleted;
 
-    QByteArray theGroup = aGroup + '\x1d';
-    QSet<QByteArray> groups;
-    groups << aGroup;
-
-    for (KEntryMap::ConstIterator entryMapIt( d->entryMap.constBegin() ); entryMapIt != d->entryMap.constEnd(); ++entryMapIt) {
-        if (entryMapIt.key().mKey.isNull() && entryMapIt.key().mGroup.startsWith(theGroup)) {
-            groups << entryMapIt.key().mGroup;
-        }
-    }
-
+    const QSet<QByteArray> groups = d->allSubGroups(aGroup);
     foreach (const QByteArray& group, groups) {
-        const QStringList keys = keyList(QString::fromUtf8(group));
+        const QStringList keys = d->keyListImpl(group);
         foreach (const QString& _key, keys) {
             const QByteArray &key = _key.toUtf8();
             if (d->canWriteEntry(group, key.constData())) {
@@ -668,11 +812,12 @@ bool KConfig::isConfigWritable(bool warnUser)
 
         // Note: We don't ask the user if we should not ask this question again because we can't save the answer.
         errorMsg += i18n("Please contact your system administrator.");
-        QString cmdToExec = KStandardDirs::findExe(QString("kdialog"));
+        QString cmdToExec = KStandardDirs::findExe(QString::fromLatin1("kdialog"));
         if (!cmdToExec.isEmpty() && componentData().isValid())
         {
-            QProcess::execute(cmdToExec,QStringList() << "--title" << componentData().componentName()
-                    << "--msgbox" << errorMsg.toLocal8Bit());
+            QProcess::execute(cmdToExec, QStringList()
+                              << QString::fromLatin1("--title") << componentData().componentName()
+                              << QString::fromLatin1("--msgbox") << errorMsg);
         }
     }
 
@@ -684,7 +829,11 @@ bool KConfig::isConfigWritable(bool warnUser)
 bool KConfig::hasGroupImpl(const QByteArray& aGroup) const
 {
     Q_D(const KConfig);
-    return d->entryMap.hasEntry(aGroup);
+
+    // No need to look for the actual group entry anymore, or for subgroups:
+    // a group exists if it contains any non-deleted entry.
+
+    return d->hasNonDeletedEntries(aGroup);
 }
 
 bool KConfigPrivate::canWriteEntry(const QByteArray& group, const char* key, bool isDefault) const
@@ -699,7 +848,7 @@ void KConfigPrivate::putData( const QByteArray& group, const char* key,
                       const QByteArray& value, KConfigBase::WriteConfigFlags flags, bool expand)
 {
     KEntryMap::EntryOptions options = convertToOptions(flags);
-    
+
     if (bForceGlobal)
         options |= KEntryMap::EntryGlobal;
     if (expand)
@@ -709,7 +858,7 @@ void KConfigPrivate::putData( const QByteArray& group, const char* key,
         options |= KEntryMap::EntryDeleted;
 
     bool dirtied = entryMap.setEntry(group, key, value, options);
-    if( flags &  KConfigBase::Persistent && dirtied )
+    if (dirtied && (flags & KConfigBase::Persistent))
         bDirty = true;
 }
 
