@@ -49,6 +49,7 @@ private Q_SLOTS:
     void testMimeTypeTraderForTextPlain();
     void testMimeTypeTraderForDerivedMimeType();
     void testMimeTypeTraderForAlias();
+    void testPreferredService();
     void testHasServiceType1();
     void testHasServiceType2();
     void testPatterns_data();
@@ -57,8 +58,14 @@ private Q_SLOTS:
     void testExtractKnownExtension();
     void testParseMagicFile_data();
     void testParseMagicFile();
+    void testHelperProtocols();
+    void testFromThread();
+
+    void testThreads();
 private:
     QList<KMimeMagicRule> m_rules;
+    QString m_nonKdeApp;
+    QString m_kdeApp;
 };
 
 
