@@ -202,6 +202,15 @@ void BrowserRun::slotBrowserScanFinished(KJob *job)
   }
 }
 
+static KMimeType::Ptr fixupMimeType (const QString& mimeType, const QString& fileName)
+{
+    KMimeType::Ptr mime = KMimeType::mimeType(mimeType);
+    if ((!mime || mime->isDefault()) && !fileName.isEmpty()) {
+        mime = KMimeType::findByUrl(fileName, 0, false, true);
+    }
+    return mime;
+}
+
 void BrowserRun::slotBrowserMimetype( KIO::Job *_job, const QString &type )
 {
     Q_ASSERT( _job == KRun::job() ); Q_UNUSED(_job)
@@ -242,6 +251,13 @@ void BrowserRun::slotBrowserMimetype( KIO::Job *_job, const QString &type )
         job->putOnHold();
         setJob( 0 );
 
+        // If the current mime-type is the default mime-type, then attempt to
+        // determine the "real" mimetype from the file name.
+        KMimeType::Ptr mimePtr = fixupMimeType(_type, suggestedFileName.isEmpty() ? url().fileName() : suggestedFileName);
+        if (mimePtr && mimePtr->name() != _type) {
+            _type = mimePtr->name();
+        }
+
         mimeTypeDetermined( _type );
     }
 }
