@@ -79,8 +79,14 @@ public:
         // we may get disconnected from the server but we don't want to try
         // to connect every time the model is requested
         if ( !m_socketConnectFailed && !localSocketClient.isConnected() ) {
-            delete localSocketModel;
+            // ###### FIXME
+            // Cannot delete the model, other threads might still be using it.
+            // With the API that returns iterators, the only way to do this right would be to
+            // use shared pointers, for refcounting the use of the model.
+            // Meanwhile, better leak (on rare occasions) than crash.
+            //delete localSocketModel;
             localSocketModel = 0;
+            localSocketClient.disconnect();
             QString socketName = KGlobal::dirs()->locateLocal( "socket", "nepomuk-socket" );
             kDebug() << "Connecting to local socket" << socketName;
             if ( localSocketClient.connect( socketName ) ) {
@@ -157,6 +163,10 @@ bool Nepomuk::MainModel::init()
     return isValid();
 }
 
+void Nepomuk::MainModel::disconnect()
+{
+    s_modelContainer->localSocketClient.disconnect();
+}
 
 Soprano::StatementIterator Nepomuk::MainModel::listStatements( const Statement& partial ) const
 {
